/*
 * sci_fifo.h
 *
 *  Created on: 2020827
 *      Author: Yeguanyong
 */

#ifndef SCI_FIFO_H_
#define SCI_FIFO_H_


#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h>
#include "DSP2833x_Device.h"     // DSP2833x Headerfile Include File
#include "DSP2833x_Examples.h"   // DSP2833x Examples Include File

#define	SCIA_FIFO_EN
#define	SCIB_FIFO_EN
#define	xSCIC_FIFO_EN
/* ˿ں */
typedef enum
{
	COMA = 0,	/* SCIA */
	COMB = 1,	/* SCIB */
	COMC = 2	/* SCIC */
}COM_PORT_E;

typedef enum
{
    LWRB_EVT_READ,                              /*!< Read event */
    LWRB_EVT_WRITE,                             /*!< Write event */
    LWRB_EVT_RESET,                             /*!< Reset event */
}LWRB_EVT;

#define BUF_IS_VALID(b)                 ((b) != NULL && (b)->buff != NULL && (b)->size > 0)
#define BUF_MIN(x, y)                   ((x) < (y) ? (x) : (y))
#define BUF_MAX(x, y)                   ((x) > (y) ? (x) : (y))
#define BUF_SEND_EVT(b, type, bp)       do { if ((b)->evt_fn != NULL) { (b)->evt_fn((b), (type), (bp)); } } while (0)


struct LWRB;
typedef void (*lwrb_evt_fn)(volatile struct LWRB *buff, LWRB_EVT evt, size_t bp);

typedef struct LWRB
{
    unsigned char* buff;                              /*!< Pointer to buffer data.
                                                    Buffer is considered initialized when `buff != NULL` and `size > 0` */
    size_t size;                                /*!< Size of buffer data. Size of actual buffer is `1` byte less than value holds */
    size_t r;                                   /*!< Next read pointer. Buffer is considered empty when `r == w` and full when `w == r - 1` */
    size_t w;                                   /*!< Next write pointer. Buffer is considered empty when `r == w` and full when `w == r - 1` */
    lwrb_evt_fn evt_fn;                         /*!< Pointer to event callback function */
}LWRB;


#ifdef SCIA_FIFO_EN
	#define SCIA_RX_BUF_SIZE	1*64
extern unsigned char sci_bufa[SCIA_RX_BUF_SIZE];
extern volatile LWRB scia_ringbuff;
interrupt void scia_rx_fifo_isr(void);
#endif
#ifdef SCIB_FIFO_EN
	#define SCIB_RX_BUF_SIZE	1*512
extern unsigned char sci_bufb[SCIB_RX_BUF_SIZE];
extern volatile LWRB scib_ringbuff;
interrupt void scib_rx_fifo_isr(void);
#endif

unsigned char     lwrb_init(volatile LWRB* buff, void* buffdata, size_t size);
unsigned char     lwrb_is_ready(volatile LWRB* buff);
void        lwrb_free(volatile LWRB* buff);
void        lwrb_reset(volatile LWRB* buff);
void        lwrb_set_evt_fn(volatile LWRB* buff, lwrb_evt_fn fn);

/* Read/Write functions */
size_t      lwrb_write(volatile LWRB* buff, const void* data, size_t btw);
size_t      lwrb_read(volatile LWRB* buff, void* data, size_t btr);
size_t      lwrb_peek(volatile LWRB* buff, size_t skip_count, void* data, size_t btp);

/* Buffer size information */
size_t      lwrb_get_free(volatile LWRB* buff);
size_t      lwrb_get_full(volatile LWRB* buff);

/* Read data block management */
void*       lwrb_get_linear_block_read_address(volatile LWRB* buff);
size_t      lwrb_get_linear_block_read_length(volatile LWRB* buff);
size_t      lwrb_skip(volatile LWRB* buff, size_t len);

/* Write data block management */
void*       lwrb_get_linear_block_write_address(volatile LWRB* buff);
size_t      lwrb_get_linear_block_write_length(volatile LWRB* buff);
size_t      lwrb_advance(volatile LWRB* buff, size_t len);

void scisendchara(unsigned char temp);
void send_buf_scia(unsigned char *buf, unsigned short int len);
void scisendcharb(unsigned char temp);
void send_buf_scib(unsigned char *buf, unsigned short int len);
void bsp_initsci(void);

#ifdef __cplusplus
}
#endif /* extern "C" */

#endif /* SCI_FIFO_H_ */
